@@ -2,8 +2,7 @@
 #include <cuda.h>
 #include <cuda_fp16.h>
 #include <curand_kernel.h>
-
-#include "3rdparty/cub/cub/cub.cuh"
+#include <cub/cub.cuh>
 
 namespace lightseq {
 namespace cuda {
@@ -13,6 +12,10 @@ const float logit_thresh_min = -64.f;
 const float min_log_probability = -2000.f;
 const float epsilon = 0.000000000001;
 
+template <typename T>
+void launch_scaled_colsum(const T* inp, T* out, int rows, int cols, float scale,
+                          cudaStream_t stream);
+
 template <typename T>
 void ker_norm_layer_launcher(int token_num, int hidden_size,
                              cudaStream_t stream, T* matrix, const T* scale,
